package init;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.henshin.interpreter.EGraph;
import org.eclipse.emf.henshin.interpreter.Engine;
import org.eclipse.emf.henshin.interpreter.RuleApplication;
import org.eclipse.emf.henshin.interpreter.impl.EGraphImpl;
import org.eclipse.emf.henshin.interpreter.impl.EngineImpl;
import org.eclipse.emf.henshin.interpreter.impl.LoggingApplicationMonitor;
import org.eclipse.emf.henshin.interpreter.impl.RuleApplicationImpl;
import org.eclipse.emf.henshin.model.Rule;
import org.eclipse.emf.henshin.model.TransformationSystem;
import org.eclipse.emf.henshin.model.resource.HenshinResourceSet;
import org.eclipse.uml2.uml.UMLPackage;

/**
 * Bank example for the Henshin interpreter. Shows the usage of the interpreter.
 * 
 * @author Christian Krause
 */
public class Example {

	/**
	 * Relative path to the model files.
	 */
	public static final String PATH = "";

	public static final LoggingApplicationMonitor monitor = new LoggingApplicationMonitor();

	/**
	 * Run the bank example.
	 * 
	 * @param path
	 *            Relative path to the model files.
	 * @param saveResult
	 *            Whether the result should be saved.
	 */
	public static void run(String path, boolean saveResult) {
		HenshinResourceSet resourceSet = new HenshinResourceSet(PATH);
		resourceSet.registerXMIResourceFactories("uml");
		UMLPackage.eINSTANCE.getName();
		// Load a model:
		EObject root = resourceSet.getObject("example-architecture-uml.uml");

		// Load a transformation system:
		TransformationSystem system = resourceSet
				.getTransformationSystem("default.henshin");

		// Prepare the engine:
		Engine engine = new EngineImpl();

		// Initialize the graph:
		EGraph graph = new EGraphImpl();
		graph.addTree(root);
		RuleApplication application = new RuleApplicationImpl(engine);
		application.setEGraph(graph);

		Rule unit = system.findRuleByName("testRule");
		application.setRule(unit);

		if (!application.execute(monitor)) {
			throw new RuntimeException("Error executing "
					+ application.getRule().getName());
		}

		EObject result = (EObject) application
				.getResultParameterValue("result");

		// Save the result:
		resourceSet.saveObject(result, "example-result.xmi");

	}

	public static void main(String[] args) {
		run(PATH, true);
	}

}